package goredis

import (
	"context"
	"fmt"
	"net"
	"runtime"
	"time"

	ctxUtil "gitlab.wanzhuangkj.com/tush/xpkg/gin/xctx"
	"gitlab.wanzhuangkj.com/tush/xpkg/logger"

	"github.com/redis/go-redis/v9"
	"go.uber.org/zap"
)

type LogHook struct {
	logger *zap.Logger
}

func NewLogHook(logger *zap.Logger) LogHook {
	return LogHook{
		logger: logger,
	}
}

func (LogHook) DialHook(next redis.DialHook) redis.DialHook {
	return func(ctx context.Context, network, addr string) (net.Conn, error) {
		return next(ctx, network, addr)
	}
}

const max = 64
const logPrefix = "redis"

func (x LogHook) ProcessHook(next redis.ProcessHook) redis.ProcessHook {
	return func(ctx context.Context, cmd redis.Cmder) error {
		st := time.Now()
		f, l, fn := caller()
		line := fmt.Sprintf("%s %s:%d ", fn, f, l)
		err := next(ctx, cmd)
		if err != nil {
			if x.logger != nil {
				x.logger.Info(Pretty(fmt.Sprintf("%s: %s", logPrefix, cmd.String()), max),
					logger.Any("err", err),
					logger.String("cost", time.Since(st).String()),
					logger.String("line", line),
					ctxUtil.CtxTraceIDField(ctx))
			}
			return err
		}
		if x.logger != nil {
			x.logger.Info(Pretty(fmt.Sprintf("%s: %s", logPrefix, cmd.String()), max),
				logger.String("cost", time.Since(st).String()),
				logger.String("line", line),
				ctxUtil.CtxTraceIDField(ctx))
		}
		return nil
	}
}
func (LogHook) ProcessPipelineHook(next redis.ProcessPipelineHook) redis.ProcessPipelineHook {
	return func(ctx context.Context, cmds []redis.Cmder) error {
		return next(ctx, cmds)
	}
}

func Pretty(str string, limit int) string {
	if len(str) > limit {
		return str[:limit] + "......"
	}
	return str
}

func caller() (string, int, string) {
	pc, file, line, ok := runtime.Caller(6)
	if ok {
		fn := runtime.FuncForPC(pc)
		return file, line, fn.Name()
	}
	return file, line, ""
}
