package wg

import (
	"context"
	ctxUtil "gitlab.wanzhuangkj.com/tush/xpkg/gin/xctx"
	"gitlab.wanzhuangkj.com/tush/xpkg/logger"
	"gitlab.wanzhuangkj.com/tush/xpkg/xerrors/xerror"
	"sync"
)

func Go(ctx context.Context, n int, fn func() error) error {
	if n <= 0 {
		return xerror.New("n cannot be less or equal then 0")
	}
	if fn == nil {
		return xerror.New("fn cannot be nil")
	}
	wg := &sync.WaitGroup{}
	wg.Add(n)
	for i := 0; i < n; i++ {
		go func(c context.Context, idx int) {
			runner(c, wg, fn)
		}(ctx, i)
	}
	wg.Wait()
	return nil
}

func runner(ctx context.Context, wg *sync.WaitGroup, fn func() error) {
	defer wg.Done()
	defer func() {
		if err := recover(); err != nil {
			logger.Error("wg.Go fn panic", logger.Any("err", err), ctxUtil.CtxTraceIDField(ctx))
		}
	}()
	err := fn()
	if err != nil {
		logger.Error("wg.Go fn err", logger.Err(err), ctxUtil.CtxTraceIDField(ctx))
	}
}
